@@ -3,6 +3,7 @@ from django.conf.urls.defaults import url
 from django.core.urlresolvers import reverse
 from django.db import router
 from django.db.models import signals
+from django.db.transaction import commit_on_success
 from django.http import HttpResponseNotFound, HttpResponse, HttpResponseBadRequest, HttpResponseRedirect
 from django.utils import simplejson
 from django.utils.translation import ugettext_lazy as _
@@ -147,18 +148,22 @@ class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmi
         return reverse('admin:{0}_{1}_moved'.format(*info))
 
 
+    @commit_on_success
     def api_node_moved_view(self, request):
         """
         Update the position of a node, from a API request.
         """
         try:
+            moved_id = long(request.POST['moved_id'])
+            target_id = long(request.POST['target_id'])
+            position = request.POST['position']
+            previous_parent_id = long(request.POST['previous_parent_id']) or None
+
             # Not using .non_polymorphic() so all models are downcasted to the derived model.
             # This causes the signal below to be emitted from the proper class as well.
-            moved = self.model.objects.get(pk=request.POST['moved_id'])
-            target = self.model.objects.get(pk=request.POST['target_id'])
-            previous_parent_id = int(request.POST['previous_parent_id']) or None
-            position = request.POST['position']
-        except KeyError as e:
+            moved = self.model.objects.get(pk=moved_id)
+            target = self.model.objects.get(pk=target_id)
+        except (ValueError, KeyError) as e:
             return HttpResponseBadRequest(simplejson.dumps({'action': 'foundbug', 'error': str(e[0])}), content_type='application/json')
         except self.model.DoesNotExist as e:
             return HttpResponseNotFound(simplejson.dumps({'action': 'reload', 'error': str(e[0])}), content_type='application/json')
@@ -166,6 +171,7 @@ class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmi
         if not self.can_have_children(target) and position == 'inside':
             return HttpResponse(simplejson.dumps({
                 'action': 'reject',
+                'moved_id': moved_id,
                 'error': _(u'Cannot place \u2018{0}\u2019 below \u2018{1}\u2019; a {2} does not allow children!').format(moved, target, target._meta.verbose_name)
             }), content_type='application/json', status=409)  # Conflict
         if moved.parent_id != previous_parent_id:
@@ -183,13 +189,19 @@ class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmi
         }[position]
         moved.move_to(target, mptt_position)
 
-        # Fire post_save signal manually, because django-mptt doesn't do that.
-        # Allow models to be notified about a move, so django-fluent-contents can update caches.
-        using = router.db_for_write(moved.__class__, instance=moved)
-        signals.post_save.send(sender=moved.__class__, instance=moved, created=False, raw=False, using=using)
+        # Some packages depend on calling .save() or post_save signal after updating a model.
+        # This is required by django-fluent-pages for example to update the URL caches.
+        # Make sure the updated version (with new parent_id/lft/rgt fields is fetched)
+        moved = self.model.objects.get(pk=moved_id)
+        moved.save()
 
         # Report back to client.
-        return HttpResponse(simplejson.dumps({'action': 'success', 'error': ''}), content_type='application/json')
+        return HttpResponse(simplejson.dumps({
+            'action': 'success',
+            'error': None,
+            'moved_id': moved_id,
+            'action_column': self.actions_column(moved),
+        }), content_type='application/json')
 
 
     def move_up_view(self, request, object_id):
